# -*- coding: utf-8 -*-
# $Id: alias.py 56146 2007-12-26 15:31:17Z glenfant $

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import INonStructuralFolder
from Products.Archetypes.public import *
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.document import finalizeATCTSchema
from Products.SimpleAlias.config import TOOL_ID

try:
    from Products.CMFDynamicViewFTI.interface import ISelectableBrowserDefault
    HAS_ISBD = True
except ImportError:
    HAS_ISBD = False

try:
    from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
    AliasReferenceWidget = ReferenceBrowserWidget
except ImportError:
    AliasReferenceWidget = ReferenceWidget


schema = ATCTContent.schema.copy() +  Schema((
    StringField('title',
                required=0,
                searchable=1,
                default='Link not set',
                accessor='Title',
                widget=StringWidget(visible={'view' : 'visible', 'edit':'visible'})),
    StringField('description',
                searchable=1,
                accessor='Description',
                widget=TextAreaWidget(visible={'view' : 'visible', 'edit':'visible'})),
    ReferenceField('alias',
                   multiValued=0,
                   relationship='linksTo',
                   allowed_types_method='getTypes',
                   vocabulary_display_path_bound=0,
                   widget=AliasReferenceWidget(size=80, checkbox_bound=1, populate=1, label='Target object',
                                               description='Select the target object this alias links to. <br/> <b>Note:</b> if a user may view this Alias then he can always see the title and description of this target object. <br />Even when he has no view permissions for this target object. Beware if you want this.',
                                               label_msgid="label_alias",
                                               description_msgid="desc_alias",
                                               force_close_on_insert=1,
                                               i18n_domain="simplealias"),
                   ),
    BooleanField('autoTitle',
                 default=1,
                 widget=BooleanWidget(label='Auto title',
                                      description="Check if you want to auto set title and description of alias with title and description of targeted object. Uncheck if you want specify another title and description for the alias.",
                                      label_msgid="label_auto_title",
                                      description_msgid="desc_auto_title",
                                      i18n_domain="simplealias")),
    BooleanField('showTarget',
                 default=1,
                 widget=BooleanWidget(label='Show target',
                                      description="Check if you want to include the target object's content while viewing this alias. Uncheck will only show the title and description.",
                                      label_msgid="label_show_target",
                                      description_msgid="desc_show_target",
                                      i18n_domain="simplealias")),
    BooleanField('showHint',
                 default=False,
                 widget=BooleanWidget(label='Show alias hint',
                                      description="Check if you want to include text 'This is alias for ....' above the renedered target object",
                                      label_msgid="label_show_hint",
                                      description_msgid="desc_show_hint",
                                      i18n_domain="simplealias")),
))

finalizeATCTSchema(schema)

class Alias(ATCTContent):
    """
An Alias is a link to another object in the portal.
    """

    schema = schema

    # FIXME: Could remove most of these since most are in GS profile. Seems that
    # only meta_type should subsist.
    global_allow = 1
    content_icon = 'alias_icon.gif'
    archetype_name = "Alias"
    portal_type = meta_type = "Alias"
    immediate_view = "alias_view"
    default_view = "alias_view"
    suppl_views = ()
    newTypeFor = ()

    _at_rename_after_creation = True
    removed = 0

    def getTypes(self, object=None):
        simplealias_tool = getToolByName(self, TOOL_ID)
        return simplealias_tool.getLinkableTypes(object)


    def DEPRECATED_generateNewId(self):
        # FIXME: Runs perfectly with AT BaseObject standard method. Why this override ?
        # Since our title is copied from the alias we need to
        # grab it form the request instead.
        plone_tool = getToolByName(self, 'plone_utils', None)
        title = self.REQUEST.get('title', None)

        if not title:
            return None

        return plone_tool.normalizeString(title)


    def Title(self):
        refObject = self.getAlias()
        titleField = self.getField('title')
        value = titleField.get(self)
        if refObject is not None:
            if self.getAutoTitle():
                if hasattr(refObject, 'title_or_id'):
                    remoteTitle = refObject.title_or_id()
                else:
                    remoteTitle  = refObject.Title()

                # only set when they are different
                # we store the title in this object because we want to be able to show the remote title even
                # if the remote object is no longer there.
                if value != remoteTitle:
                    self.setTitle(remoteTitle)
                    self.reindexObject()
            self.removed=0
        else:
            if value!='Link not set':
                self.removed=1
                return 'Removed: ' + value
            else:
                self.removed=0
                return value
        return titleField.get(self)

    def Description(self):
        refObject = self.getAlias()
        descField = self.getField('description')
        value = descField.get(self)

        if refObject is not None:
            if self.getAutoTitle():
                remoteDescr = refObject.Description()

                if value != remoteDescr:
                    self.setDescription(remoteDescr)
                    self.reindexObject()
        else:
            if self.removed==1:
                return 'Object is removed or deleted.'
        return descField.get(self)


    def getIcon(self, relative_to_portal=0):
        refObject = self.getAlias()
        if refObject is not None:
            simplealias_tool = getToolByName(self, TOOL_ID)
            return simplealias_tool.getAliasIcon(refObject)
        else:
            return 'alias_icon.gif'

    def getIsPrincipiaFolderish(self):
        refObject = self.getAlias()
        if refObject is not None:
            return (refObject.isPrincipiaFolderish
                    and not INonStructuralFolder.providedBy(refObject))
        else:
            return False

    # zegor : set folderish to false to solve folder_contents tab problem
    # isPrincipiaFolderish=ComputedAttribute(getIsPrincipiaFolderish, 1)
    isPrincipiaFolderish = property(getIsPrincipiaFolderish)

    def listFolderContents(self, spec=None, contentFilter=None, suppressHiddenFiles=0 ):
        return []


    def hasPermission(self):
        refObject = self.getAlias()
        if refObject is not None:
            if self.portal_membership.checkPermission('View', refObject):
                return 1 # user has permission
            else:
                return 0 # user has no permission
        else:
            if self.removed==1:
                return -1 # referenced object does not exist anymore
            else:
                return -2 # reference is not set yet

    def getTargetObjectLayout(self, target):
        """
        Returns target object 'view' action page template
        """
        if HAS_ISBD and ISelectableBrowserDefault.providedBy(target):
            return target.getLayout()
        else:
            view = target.getTypeInfo().getActionById('view')
            # If view action is view, try to guess correct template
            if view == 'view':
                view = target.portal_type.lower() + '_view'
            return view

    def targetMainMacro(self):
        """The macro that renders the central stuff of the body,
        or False if no"""

        target = self.getAlias()
        if HAS_ISBD and ISelectableBrowserDefault.providedBy(target):
            layout = target.getLayout()
        else:
            portal_types = geMultiAdapter((self, self.REQUEST), name=u'plone_tools').types()
            type_info = portal_types.getTypeInfo(target)
            try:
                view_action = type_info.getActionInfo('object/view')
                layout = view_action['url'].split('/')[-1] or getattr(type_info, 'default_view', 'view')
            except:
                # We can't have a layout
                return False
        return target.restrictedTraverse(layout).macros.get('main', False)



registerType(Alias)
