# -*- coding: utf-8 -*-
# $Id: testAliasCreation.py 56146 2007-12-26 15:31:17Z glenfant $

from Testing import ZopeTestCase
import os
from Products.SimpleAlias.tests import SimpleAliasTC
from DateTime import DateTime
from Products.Archetypes.utils import shasattr

# A test class defines a set of tests
class TestAliasCreation(SimpleAliasTC.SimpleAliasTestCase):

    # The afterSetUp method can be used to define test class variables
    # and perform initialisation before tests are run. The beforeTearDown()
    # method can also be used to clean up anything set up in afterSetUp(),
    # though it is less commonly used since test always run in a sandbox
    # that is cleared after the test is run.
    def afterSetUp(self):
        self.membership = self.portal.portal_membership
        self.membership.memberareaCreationFlag = 1
        self.addMember('fred', 'Fred Flintstone', 'fred@bedrock.com', ['Member', 'Manager'], '2002-01-01')
        self.login('fred')
        self.portal.invokeFactory('Folder', 'f1')
        self.f1 = self.portal.f1


    def addMember(self, username, fullname, email, roles, last_login_time):
        self.membership.addMember(username, 'secret', roles, [])
        member = self.membership.getMemberById(username)
        member.setMemberProperties({'fullname': fullname, 'email': email,
                                    'last_login_time': DateTime(last_login_time),})

    def testAddAlias(self):
        self.f1.invokeFactory('Alias', 'alias')
        self.failUnless(shasattr(self.f1, 'alias'))

    def testAliasEdit(self):
        self.f1.invokeFactory('Alias', 'alias')
        self.f1.invokeFactory('Document', 'doc') # used for the target
        doc   = self.f1.doc
        alias = self.f1.alias

        doc.setTitle('target doc')
        doc.setDescription('target description')
        alias.setShowTarget(False)
        alias.setShowHint(False)

        self.failUnlessEqual(alias.getShowTarget(), False)
        self.failUnlessEqual(alias.getShowHint(), False)

        # first with autoTitle=True
        alias.setTitle('some title')
        alias.setDescription('some description')
        alias.setAutoTitle(True)

        # without an alias it should show this:
        self.failUnlessEqual(alias.getAutoTitle(), True)
        self.failUnlessEqual(alias.Title(), 'Removed: some title')
        self.failUnlessEqual(alias.Description(), 'Object is removed or deleted.')

        # now set an alias
        alias.setAlias(doc)
        self.failUnlessEqual(alias.Title(), 'target doc')
        self.failUnlessEqual(alias.Description(), 'target description')

        # now set autoTitle = False
        alias.setAutoTitle(False)
        alias.setTitle('some title')
        alias.setDescription('some description')
        self.failUnlessEqual(alias.getAutoTitle(), False)
        self.failUnlessEqual(alias.Title(), 'some title')
        self.failUnlessEqual(alias.Description(), 'some description')


    def testGetIcon(self):
        self.f1.invokeFactory('Alias', 'alias')
        self.f1.invokeFactory('Document', 'doc') # used for the target
        doc   = self.f1.doc
        alias = self.f1.alias
        alias.setAlias(doc)

        self.failUnlessEqual(alias.getIcon(), 'document_icon_alias.gif')

    def testTargetObjectLayout(self):
        self.f1.invokeFactory('Alias', 'alias')
        self.f1.invokeFactory('Document', 'doc') # used for the target
        doc   = self.f1.doc
        alias = self.f1.alias
        alias.setAlias(doc)
        macro = alias.targetMainMacro()
        sourcefile = dict(macro)['setSourceFile']
        self.failUnlessEqual(os.path.basename(sourcefile), 'document_view.pt')

# This boilerplate method sets up the test suite
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # Add our test class here - you can add more test classes if you wish,
    # and they will be run together.
    suite.addTest(makeSuite(TestAliasCreation))
    return suite
